package lxhttp

import (
	"bytes"
	"crypto/tls"
	"encoding/base64"
	Rsvmp "git.listensoft.net/tool/jspkit/common/rsvmp"
	"git.listensoft.net/tool/jspkit/logger"
	"git.listensoft.net/tool/jspkit/taxerr"
	"go.uber.org/zap"
	"golang.org/x/net/publicsuffix"
	"io"
	"net/http"
	"net/http/cookiejar"
	"net/url"
	"strings"
	"time"
)

// PostByteReader PostReader 不规则参数 直接传字符串
func PostByteReader(client *http.Client, uri string, bytesData []byte, headers map[string]string, Cookies []*http.Cookie) ([]byte, error) {
	f := 0
	var ck []*http.Cookie
beggin:
	if len(ck) != 0 {
		Cookies = append(Cookies, ck...)
	}
	request, err := http.NewRequest("POST", uri, bytes.NewReader(bytesData))
	if err != nil {
		return nil, err
	}
	if strings.Contains(uri, "mhzx/api/mh/mhsy/getUserInfo") {
		request, err = http.NewRequest("GET", uri, bytes.NewReader(bytesData))
		if err != nil {
			return nil, err
		}
	}
	request.Header.Add("Content-Type", "application/json")
	request.Header.Add("User-Agent", "Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36")
	for k, v := range headers {
		request.Header.Set(k, v)
	}
	// 瑞数加密
	for _, v := range Cookies {
		request.AddCookie(v)
	}
	resp, err := client.Do(request)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()
	if !strings.Contains(uri, "api/login") {
		if resp.StatusCode == 412 && f < 3 && Rsvmp.NeedRsvmpAPi(uri) {
			ck, uri = Rsvmp.Rsvmp(uri)
			f++
			goto beggin
		}
		if resp.StatusCode != 200 {
			logger.Info("resp", zap.String("uri", uri), zap.Int("code", resp.StatusCode))
			body1, _ := io.ReadAll(resp.Body)
			return body1, taxerr.NewWebStuckTitle(true)
		}
	}
	body, err := io.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}
	if strings.Contains(resp.Header.Get(`Content-Encoding`), "gzip") {
		return GZIPDecode(body)
	}
	if strings.Contains(string(body), "目前暂不支持查看该申报表的申报明细") {
		return nil, taxerr.NewSystemV3("税局提示:目前暂不支持查看该申报表的申报明细", "请核实")
	}
	if strings.Contains(uri, "pdf") || strings.Contains(uri, "download") || strings.Contains(uri, "Pdf") || strings.Contains(uri, "exportSbmxxqcx") || strings.Contains(uri, `previewPdfData`) || strings.Contains(uri, "DescribeSbmxxqcx") {
		logger.Info("xx", zap.String("uri", uri), zap.Int("pdf字节changed", len(body)))
	} else {
		logger.Info("xx", zap.String("uri", uri), zap.Int("body", len(body)))
	}
	return body, nil
}

func NewHttpClientForNoRedirects(proxy, auth string) (client *http.Client) {
	// 禁止重定向
	client = &http.Client{
		CheckRedirect: func(req *http.Request, via []*http.Request) error {
			// 返回错误以阻止重定向
			return http.ErrUseLastResponse
		},
		Timeout: 10 * time.Second,
	}
	tr := &http.Transport{
		TLSClientConfig: &tls.Config{InsecureSkipVerify: true}, //忽略证书校验
	}
	if proxy != "" {
		u, _ := url.Parse(proxy)
		tr.Proxy = http.ProxyURL(u)
	}
	if auth != "" {
		tr.ProxyConnectHeader = http.Header{
			"Proxy-Authorization": []string{"Basic " + base64.StdEncoding.EncodeToString([]byte(auth))},
		}
	}
	client.Transport = tr
	options := cookiejar.Options{
		PublicSuffixList: publicsuffix.List,
	}
	curCookieJar, _ := cookiejar.New(&options)
	client.Jar = curCookieJar
	return
}

func AutoHttp(client *http.Client, uri string, Method string, bytesData []byte, headers map[string]string) (body []byte, err error) {
	var Cookies []*http.Cookie
	for range "..." {
		request, err := http.NewRequest(Method, uri, bytes.NewReader(bytesData))
		if err != nil {
			continue
		}
		for k, v := range headers {
			request.Header.Add(k, v)
		}
		// 瑞数加密
		for _, v := range Cookies {
			request.AddCookie(v)
		}
		resp, err := client.Do(request)
		if err != nil {
			continue
		}
		if resp.StatusCode == 412 {
			logger.Info("需要瑞数加密了：", zap.Int("code", resp.StatusCode))
			Cookies, _ = Rsvmp.Rsvmp(uri)
			logger.Info("加密完成")
			logger.Info("ck", zap.Any("ck", Cookies))
			continue
		}
		defer resp.Body.Close()
		body, err = io.ReadAll(resp.Body)
		if err != nil {
			continue
		}
		break
	}
	return
}
